/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.io.InputStream;
import java.io.InputStreamReader;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApi;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.session.Session;

/**
 * {@link PGPSessionKeyExtractor} uses an OX Guard endpipoint to extract "Public-Key Encrypted Session Key Packets" from a given PGP input stream.
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.10.0
 * @see RFC 4880 - "OpenPGP Message Format"
 */
public class PGPSessionKeyExtractor extends AbstractGuardAccess{

    private static String EXTRACT_SESSION_KEYS_ACTION = "extractsessionkeys";

    /**
     * Extracts the encrypted session packets from the given PGP data.
     *
     * @param inputStream The PGP data to extract the encrypted session key packets from
     * @return
     * @throws OXException
     */
    public String[] getEncryptedSession(InputStream inputStream, Session session) throws OXException {

        GuardApi guardApi = requireGuardApi(PGPCORE_ENDPOINT);
        InputStream response = guardApi.processResource(
            GuardApis.mapFor("action",EXTRACT_SESSION_KEYS_ACTION),
            inputStream,
            "application/octet-stream",
            "data",
            session);

        JSONObject result = parseResult(response);
        JSONArray sessionKeyPackets;
        try {
            JSONObject data = result.getJSONObject("data");
            sessionKeyPackets = data.getJSONArray("encryptedSessionKeyPackets");
            String[] ret = new String[sessionKeyPackets.length()];
            for (int i=0; i < sessionKeyPackets.length(); i++) {
                ret[i] = sessionKeyPackets.getString(i);
            }
            return ret;
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }

    /**
     * Parses the JSON result
     *
     * @param response The result to parse
     * @return The parsed JSON result
     * @throws OXException
     * @throws JSONException
     */
    private JSONObject parseResult(InputStream response) throws OXException {
        try {
            return new JSONObject(new InputStreamReader(response));
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }
}
