/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.internal.authentication;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.List;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.ldap.User;
import com.openexchange.guard.api.Cookie;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.api.Header;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.GuardApiImpl;
import com.openexchange.guard.osgi.Services;
import com.openexchange.session.Session;
import com.openexchange.user.UserService;

/**
 * {@link GuardLoginHandler} performs a login request against the OX Guard service on behalf of a given user.
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.4
 */
public class GuardLoginHandler extends AbstractGuardAccess {

    private final String AUTH_TOKEN = "auth";

    /**
     * Internal method to check if we got a valid auth-token from the Guard server
     *
     * @param authToken The authToken to check for validity
     * @return true if the authToken seems to be valid, false otherwise.
     */
    private boolean isAuthTokenValid(String authToken) {
        if(authToken != null && authToken.length() > 20 && authToken.contains("=")) {
            return true;
        }
        return false;
    }

    /**
     * Check if Guard reporting lockout
     * @param authToken
     * @return
     */
    private boolean isLockout(String authToken) {
        return (authToken != null && authToken.equals("Lockout"));
    }

    private boolean isPin(JSONObject resp) {
        try {
            if (resp.has("pin") && resp.getBoolean("pin")) return true;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return false;
    }

    private GuardAuthenticationToken loginInternal(JSONObject json, String guardSessionToken, Session session, List<Cookie> cookies, String userAgent) throws OXException {
        GuardApiImpl guardApi = requireGuardApi(LOGIN_ENDPOINT);
        try {
            //Perform Guard login on behalf of the user
            InputStream responseStream = guardApi.doCallSessionSensitivePost(
                GuardApis.mapFor("action", "login", "session", session.getSessionID()),
                json,
                InputStream.class,
                session,
                cookies,
                Arrays.asList(new Header[] {
                    new Header("User-Agent", userAgent),
                    new Header("X-Forwarded-For", session.getLocalIp())
                }));

            //Parse the response
            JSONObject response = new JSONObject(new InputStreamReader(responseStream));
            if(response.has(AUTH_TOKEN)) {
                final String authToken = response.getString(AUTH_TOKEN);
                if(isAuthTokenValid(authToken)) {
                    return new GuardAuthenticationToken(guardSessionToken, authToken);
                }
                if(isLockout(authToken)) {
                    throw GuardExceptionCodes.LOCKOUT.create();
                }
                else {
                    if (authToken != null && authToken.startsWith("Error")) {
                        throw GuardExceptionCodes.UNKNOWN_ERROR.create(authToken);
                    }
                    //invalid login
                    return null;
                }
            }
            else {
                if (isPin(response)) {
                    throw GuardExceptionCodes.BLOCKED_BY_PIN.create();
                } else {
                    throw GuardExceptionCodes.MISSING_AUTH.create();
                }
            }
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }

    public GuardAuthenticationToken loginWithSymmetricKey(Session session, String guardSessionToken, String key, List<Cookie> cookies, String userAgent) throws OXException {
        try {
            JSONObject json = new JSONObject();
            if (key != null) {
                json.put("pgp_session_key", key);
            }
            return loginInternal(json, guardSessionToken, session, cookies, userAgent);
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }

    /**
     * Performs a login against the OX Guard server on behalf of the given session's user.
     *
     * @param session The session associated to the user which will be logged in into OX Guard
     * @param guardSessionToken The session value which will be used
     * @param password The OX Guard password
     * @param cookies user's client cookies
     * @param userAgent The client's user agent
     * @return An {@link GuardAuthenticationToken} if the login was successful, null otherwise
     * @throws OXException
     */
    public GuardAuthenticationToken login(Session session, String guardSessionToken, String password, String ePassword, List<Cookie> cookies, String userAgent) throws OXException {
        try {
            JSONObject json = new JSONObject();
            if (ePassword != null) {
                json.put("e_encr_password", ePassword);
            }
            if (password != null) {
                json.put("encr_password", password);
            }
            UserService userService = Services.getService(UserService.class);
            User user = userService.getUser(session.getUserId(), session.getContextId());
            json.put("email", user.getMail());
            json.put("guest", user.getCreatedBy() > 0);
            return loginInternal(json, guardSessionToken, session, cookies, userAgent);
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }
}
