/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl;

import com.openexchange.exception.OXException;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.mail.api.DelegatingMailAccess;
import com.openexchange.mail.api.IMailFolderStorage;
import com.openexchange.mail.api.IMailMessageStorage;
import com.openexchange.mail.api.MailAccess;
import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
import com.openexchange.session.Session;

/**
 * {@link GuardMailAccess}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.4
 */
public class GuardMailAccess extends DelegatingMailAccess {

    private static final long serialVersionUID = 3968669814403573518L;

    private IMailMessageStorage messageStorage;
    private final PGPMailRecognizer pgpMailRecognizer;
    private final GuardAuthenticationToken authToken;

    /**
     * Initializes a new {@link GuardMailAccess}.
     *
     * @param delegate The {@link MailAccess} to decorate
     * @param session The session
     * @param pgpMailRecognizer A strategy for recognizing PGP mails.
     * @param authToken The authentication token used for decrypting emails.
     */
    public GuardMailAccess(MailAccess<IMailFolderStorage, IMailMessageStorage> delegate, Session session, PGPMailRecognizer pgpMailRecognizer, GuardAuthenticationToken authToken) {
        super(delegate, session);
        this.pgpMailRecognizer = pgpMailRecognizer;
        this.authToken = authToken;
    }

    /**
     * Initializes a new {@link GuardMailAccess}.
     *
     * @param delegate The {@link MailAccess} to decorate.
     * @param session The session
     * @param accountId The account ID
     * @param pgpMailRecognizer A strategy for recognizing PGP mail.
     * @param authToken
     */
    public GuardMailAccess(MailAccess<IMailFolderStorage, IMailMessageStorage> delegate, Session session, int accountId, PGPMailRecognizer pgpMailRecognizer, GuardAuthenticationToken authToken) {
        super(delegate, session, accountId);
        this.pgpMailRecognizer = pgpMailRecognizer;
        this.authToken = authToken;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.mail.api.DelegatingMailAccess#getMessageStorage()
     */
    @Override
    public IMailMessageStorage getMessageStorage() throws OXException {
        if (messageStorage == null) {
            messageStorage = new GuardMessageStorage(session, super.getMessageStorage(), pgpMailRecognizer, authToken);
        }
        return messageStorage;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.mail.api.DelegatingMailAccess#isCacheable()
     */
    @Override
    public boolean isCacheable() {
        //MailAccess instances are cached by user and not by session;
        //Non caching prevents a request to obtain a cached MailAccess for the same user but with another session (containing a guard auth-token).
        return false;
    }
}
